/*
 * Copyright (C) 1999,2000  Ross Combs (rocombs@cs.nmsu.edu)
 * Copyright (C) 1999,2000  Rob Crittenden (rcrit@greyoak.com)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
#include "common/setup_before.h"
#ifdef HAVE_STDDEF_H
# include <stddef.h>
#else
# ifndef NULL
#  define NULL ((void *)0)
# endif
#endif
#include <compat/socket.h>
#include "common/packet.h"
#include "common/file_protocol.h"
#include "common/eventlog.h"
#include "connection.h"
#include "common/queue.h"
#include "file.h"
#include "common/bn_type.h"
#include "common/field_sizes.h"
#include "handle_file.h"
/* ADDED BY UNDYING SOULZZ 4/3/02 */
#ifdef HAVE_SYS_TYPES_H
# include <sys/types.h>
#endif
#ifdef HAVE_SYS_SOCKET_H
# include <sys/socket.h>
#endif
#include "compat/psock.h"
#include "common/setup_after.h"


extern int handle_file_packet(t_connection * c, t_packet const * const packet)
{
    if (!c)
    {
	eventlog(eventlog_level_error,"handle_file_packet","[%d] got NULL connection",conn_get_socket(c));
	return -1;
    }
    if (!packet)
    {
	eventlog(eventlog_level_error,"handle_file_packet","[%d] got NULL packet",conn_get_socket(c));
	return -1;
    }
/* REMOVED BY UNDYING SOULZZ 4/3/02 */
/*
    if (packet_get_class(packet)!=packet_class_file)
    {
        eventlog(eventlog_level_error,"handle_file_packet","[%d] got bad packet (class %d)",conn_get_socket(c),(int)packet_get_class(packet));
        return -1;
    }
 */   
    switch (conn_get_state(c))
    {
    case conn_state_connected:
	switch (packet_get_type(packet))
	{
	case CLIENT_FILE_REQ:
	{
	    char const * rawname;
	    
	    if (!(rawname = packet_get_str_const(packet,sizeof(t_client_file_req),MAX_FILENAME_STR)))
	    {
		eventlog(eventlog_level_error,"handle_file_packet","[%d] got bad FILE_REQ (missing or too long filename)",conn_get_socket(c));
		
		return -1;
	    }
	    
	    file_send(c,rawname,
		      bn_int_get(packet->u.client_file_req.adid),
		      bn_int_get(packet->u.client_file_req.extensiontag),
		      bn_int_get(packet->u.client_file_req.startoffset),
		      1);
	}
	break;

	/* ADDED BY UNDYING SOULZZ 4/3/02 */
	case CLIENT_WAR3113_FILE_REQ1:
	{
	    t_packet * rpacket = NULL;
	    eventlog(eventlog_level_info,"handle_file_packet","[%d] packet processed", conn_get_socket(c) );
	    if((rpacket = packet_create(packet_class_raw))) {
		    packet_set_size(rpacket,sizeof(t_server_file_unknown1));
		    bn_short_set( &rpacket->u.server_file_unknown1.unknown1, 0x84e7 );
		    bn_short_set( &rpacket->u.server_file_unknown1.unknown2, 0x41b4 );
		    conn_push_outqueue(c, rpacket );
		    packet_del_ref( rpacket );
	    }
	    if((rpacket = packet_create(packet_class_raw))) {
		    packet_set_size(rpacket,sizeof(t_client_war3113_file_req));
		    conn_push_inqueue( c, rpacket );
		    packet_del_ref( rpacket );
	    }
	}
	break;

	/* ADDED BY UNDYING SOULZZ 4/3/02 */
	case CLIENT_WAR3113_FILE_REQ2:
	{
	    char rawname[MAX_FILENAME_STR];
	    
	    eventlog( eventlog_level_info,"handle_file_packet","[%d] new file case",conn_get_socket(c));
	    psock_recv( conn_get_socket(c), rawname, MAX_FILENAME_STR, 0 );
	
	    file_send(c, rawname, 0, 0, 0, 1);
	}
	break;	
	
	default:
	    eventlog(eventlog_level_error,"handle_file_packet","[%d] unknown file packet type 0x%04x, len %u",conn_get_socket(c),packet_get_type(packet),packet_get_size(packet));
	    
	    break;
	}
	break;
	
    default:
	eventlog(eventlog_level_error,"handle_file_packet","[%d] unknown file connection state %d",conn_get_socket(c),(int)conn_get_state(c));
    }
    
    return 0;
}
